// dcf.c
/*----------------------------------------------------------------------*/
/*! \file 
--------------------------------------------------------------------------

  	<h3> DCF.C </h3>
  	Version	1.0<br>
  	Revision	$Revision: 1.5 $<br>
 	Copyright 	Thomas Wedemeyer, Email: avr@thomas-wedemeyer.de<br> 
  	Datum		$Date: 2011/08/15 21:16:50 $<br>

	<h3>Routinen fr den Empfang des DCF-Signals und der Auswertung </h3>
	Der DCF-Impuls des Empfngermoduls wird ber den externen IRQ0 des
	ATMega detektiert.Die Lnge des DCF-Impulses wird ber den Timer2 im
	1ms-Raster gemessen.
	Um Strimpulse zu erkennen und zu verwerfen wird die Lnge und der 
	zeitliche Abstand zum Vorimpuls berprft. Bei fehlerhaften Impulsen
	wird die Aufzeichung der Bits zurckgesetzt.
	Die gltigen DCF-Impulse werden sortiert in ein Byte-Array gespeichert.
	Wurde ein DCF-Telegramm	vollstndig empfangen werden die Daten 
	ausgewertet und dabei das Parity-Bit ausgewertet.   <br>

/*----------------------------------------------------------------------*/

#include <inttypes.h>
#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include "define.h"
#include "dcf.h"


/*----------------------------------------------------------------------*/
//	Debug Variablen
/*----------------------------------------------------------------------*/
uint8_t		debug = 0;
uint16_t	db_tcnt2 = 0;

/*----------------------------------------------------------------------*/
//	Einbinden externe Strukturen
/*----------------------------------------------------------------------*/

extern volatile struct dcf_format	dcf;	//!< Controllstruktur fr DCF
extern volatile struct time_format	time;	//!< Zeitstruktur

/*----------------------------------------------------------------------*/
//! Initialisierung der DCF-Funktion
/*----------------------------------------------------------------------*/
void init_dcf()
{
	uint8_t	i;

	// DCR-Data-Array lschen
	for(i=0; i < 8; i++)
	{
		dcf.data[i] = 0;
	}

	dcf.sec = 0;
	dcf.bitcnt = 0;
	dcf.tcnt = 0;



	#ifdef DCF_INV	// Invertiertes DCF-Signal
	MCUCR	|= (1<<ISC01);				// Int0 = Falling Edge
	GICR	|= (1<<INT0);				// Int0 = IRQ Enable;
	#else
	// Externen IRQ freigeben
	MCUCR	|= (1<<ISC01) | (1<<ISC00);	// Int0 = Rising Edge
	GICR	|= (1<<INT0);				// Int0 = IRQ Enable;
	#endif


}


/*----------------------------------------------------------------------*/
//! Interruptservice-Routine IRQ0
//! Einlesen der einzelnen DCF-Bits
/*----------------------------------------------------------------------*/
SIGNAL (INT0_vect)
{
	uint8_t		i;


	#ifdef DCF_INV
	if((MCUCR & (1<<ISC00))==0)			// Falling Edge
	#else
	if(MCUCR & (1<<ISC00))			// Rising Edge
	#endif
	{
		//dcf.tcnt = TCNT1;			// Sichern des aktuellen Zhlerstandes

		if(dcf.msec > 1700)			// Mehr als 1.7Sek. vergangen!
		{
			UDR = 'N';
			dcf.bitcnt = 0;	// Neusynchronistation
			dcf.sec = 0;
			for(i=0; i < 8; i++)
			{
				dcf.data[i] = 0;
			}

			if(dcf.okay == 1)	// Wenn letzter Datensatz okay war,
			{					// dann stelle Uhrzeit
				TCNT1 = 0;		// interne Sekundenbasis zurcksetzen
				time.seconds = 60;
				time.minutes = dcf.min;
				time.hours = dcf.hour;
				time.dcf_set = 1;
				dcf.okay = 0;
			}
		}

		if(dcf.msec > 800)		// Zeitspanne zwischen den IRQ lang bei mind. 800ms,
		{
			// dann gltige Flanke!
			#ifdef DCF_INV
			MCUCR |=(1<<ISC00);		// Rising Edge
			#else					
			MCUCR &=~(1<<ISC00);	// Fallende Flanke
			#endif
		}

		dcf.msec = 0;			// Zeitmessung fr neues Bit starten

	}
	else									// Fallende Flanke
	{
		if((dcf.msec > 50) & (dcf.msec < 130))	// Grer 70ms, kleiner 130ms -> 0
		{		 
		
			UDR = '0';	
			if(dcf.bitcnt > 58)		// hier geht was schief...
			{

			}
			else 
			{
				dcf.bitcnt++;		// Erhhe nur den Bitzhler...
			}
		}
		else if ((dcf.msec > 150) & (dcf.msec < 230)) // Zeitspanne fr eine 1
		{
			UDR = '1';
			if(dcf.bitcnt > 58)	// Hier geht was schief..
			{
			
			}
			else if(dcf.bitcnt == 58)	// Einsortieren der Daten		
				dcf.data[7] = 0x01;		// Paritt Datum speichern
			else if (dcf.bitcnt > 49)	// Jahr
				dcf.data[6] |= ( 1 << (dcf.bitcnt - 50));
			else if (dcf.bitcnt > 44)	// Monat
				dcf.data[5] |= (1 << (dcf.bitcnt - 45));
			else if (dcf.bitcnt > 41)	// Wochentag
				dcf.data[4] |= (1 << (dcf.bitcnt - 42));
			else if (dcf.bitcnt > 35)	// Monatstag
				dcf.data[3] |= (1 << (dcf.bitcnt - 36));
			else if (dcf.bitcnt > 28)	// Stunden
				dcf.data[2] |= (1 << (dcf.bitcnt - 29));
			else if (dcf.bitcnt > 20)	// Monat
				dcf.data[1] |= (1 << (dcf.bitcnt - 21));
			else if (dcf.bitcnt > 14)	// Monat
				dcf.data[0] |= (1 << (dcf.bitcnt - 15));
			dcf.bitcnt++;
		}
		else	// Fehlerhafter Impuls
		{
			UDR = 'I';
			dcf.bitcnt = 0;	// Neusynchronistation
			dcf.sec = 0;
			for(i=0; i < 8; i++)
			{
				dcf.data[i] = 0;
			}

			debug = 1;
			db_tcnt2 = dcf.msec;
		}


 
		if(dcf.bitcnt > 58)	// Nachricht vollstndig!
		{
			dcf.new = 1;
		}


		#ifdef DCF_INV
		MCUCR |=(1<<ISC01);
		MCUCR &= ~(1<<ISC00);
		#else
		MCUCR	|= (1<<ISC01) | (1<<ISC00);	// Int0 = Rising Edge
		#endif
		dcf.sec = 0;
	} 
}

/*----------------------------------------------------------------------*/
//! \fn Berechnung der geraden Paritt
//! @param	value	Byte ber das die Paritt berechnet werden soll
//! @return	Paritt des Bytes
/*----------------------------------------------------------------------*/
uint8_t	check_parity(uint8_t value)
{
	uint8_t	even_parity = 0;

	while(value)
	{
		even_parity ^= (value & 0x01);
		value >>= 1; 
	};
	return even_parity;

}

/*----------------------------------------------------------------------*/
//! \fn Umwandlung der BCD-Zahl in DezimalZahl
//! @param	value	2-Stellige BCD-Zahl
//!	@return	Dezimalkodierte Zahl
/*----------------------------------------------------------------------*/
uint8_t bcd2dez(uint8_t value)
{
	uint8_t dez;

	dez = ((value & 0xF0)>> 4) * 10;
	dez += (value & 0x0F);

	return dez;
}

/*----------------------------------------------------------------------*/
//! \fn berprfung und Auswertung des DCF-Telegramms
/*----------------------------------------------------------------------*/
void check_dcf()
{
	uint8_t parity;

	if(dcf.new == 1)
	{
		dcf.new = 0;
		dcf.okay = 0;

		parity = dcf.data[1] >> 7;
		
		if(check_parity(dcf.data[1]&0x7F) != parity)
		{
			printf("Parity Error Min\r\n");
			return;	// Parittsfehler!
		}

		dcf.min = bcd2dez(dcf.data[1] & 0x7F);	// Umwandlung Minuten
		
		parity = (dcf.data[2] & 0x40) >> 6;

		if(check_parity(dcf.data[2]&0x3F) != parity)
		{
			printf("Parity Error H\r\n");
			return;	// Parittsfehler!
		}			

		dcf.hour = bcd2dez(dcf.data[2] & 0x3F);	// Umwandlung Minuten

		// Auswertung Paritt Datum
		parity = check_parity(dcf.data[3]);		// Berechne Parity Kal-Day
		parity |= (check_parity(dcf.data[4])) << 1;	// Weekday
		parity |= (check_parity(dcf.data[5])) << 2;	// Month
		parity |= (check_parity(dcf.data[6])) << 3;	// Weekday

		parity = check_parity(parity);

		if(dcf.data[7] != parity)
		{
			printf("Parity Error Cal\r\n");
			return;	// Parittsfehler!
		}

		dcf.day = bcd2dez(dcf.data[3]);
		dcf.wkday = bcd2dez(dcf.data[4]);
		dcf.month = bcd2dez(dcf.data[5]);
		dcf.year = bcd2dez(dcf.data[6]);

		dcf.okay = 1;	// Fertig entschlsselt!

		printf("\r\nDCF-Time: %2d:%2d  %2d.%2d.%2d WK: %d\r\n",	// Debugausgabe
					dcf.hour,
					dcf.min,
					dcf.day,
					dcf.month,
					dcf.year,
					dcf.wkday);

	}

	if(debug == 1)		// Debugausgabe...
	{
		debug = 0;
		printf("db_tcnt2 [msec] = %d \r\n",db_tcnt2);
	};

}






