// Timer.c
/*----------------------------------------------------------------------*/
/*! \file 
--------------------------------------------------------------------------

  	<h3> Timer.C </h3>
  	Version	1.0<br>
  	Revision	$Revision: 1.4 $<br>
 	Copyright 	Thomas Wedemeyer, Email: avr@thomas-wedemeyer.de <br>
  	Datum		$Date: 2011/08/15 21:17:03 $<br>

	<h3>Funktionen fr den Timerzugriff</h3>
  	Der 16-Bit Timer1 liefert einen "Hilfssekunden"-Takt, der fr die 
	Berechung der Uhrzeit benutzt wird, wenn das DCF-Signal ungltig ist.
	Wenn ein gltiges DCF-Signal empfangen wurde, wird der "Sekundenzhler"
	von Timer1 mit dem DCF-Sekunden synchronisiert.<br>

	Der 8-Bit Timer 2 steuert die Ausgabe auf der LED-Matix. Die Taktrate 
	des Zeilenwechsels betrgt 1ms.<br>

/*----------------------------------------------------------------------*/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
//#include <avr/signal.h>
#include "define.h"
#include "dcf.h"


/*----------------------------------------------------------------------*/
//! Einbinden externer Variablen
/*----------------------------------------------------------------------*/
extern volatile uint8_t	row[];
extern volatile uint8_t 	LED_ROW[];
extern volatile uint8_t	tick;
extern volatile struct dcf_format	dcf;	// Einbinden de DCF-Controllstruktur

/*----------------------------------------------------------------------*/
//! Initialisierung Timer 1 
//! 16-Bit Timer fr Sekundentakt
/*----------------------------------------------------------------------*/
void init_timer1()
{
	OCR1A = 31250;			// 8MHz / 256 -> 1Sekunde
	TCCR1A = 0;		// Clear Timer on Compare-Mode
	TCCR1B = (1<<CS12) | (1<<WGM12);		// Teiler 256, Clear Timer on Compare-Mode
	TIMSK |= (1<< OCIE1A);	// Enable Interrupt Timer Compare 1
}

/*----------------------------------------------------------------------*/
//! Interruptservice Routine -> Sekundentakt weiterzhlen
/*----------------------------------------------------------------------*/
SIGNAL (TIMER1_COMPA_vect )
{
	tick++;
}


/*----------------------------------------------------------------------*/
//! Initialisierung Timer 2
//! Taktgeber Zeilenwechsel
/*----------------------------------------------------------------------*/
void init_timer2()
{
	OCR2 = 125;	// Reloadwert Timer 2
	TCCR2 = (1<<WGM21) | (1<<CS22); // Clear Timer Mode, Takt= 8MHz/64
	TIMSK |= (1 << OCIE2);	// Interrupt Enable Timer Compare
}


/*----------------------------------------------------------------------*/
//! Interrupt-Serviceroutine Timer 2 CompareCatch
//! Ansteuerung LED-Matirx ber Timer 2
/*----------------------------------------------------------------------*/
SIGNAL (TIMER2_COMP_vect)
{
	// Ausgabe-Funktion fr die LED-Matrix
	const uint8_t mask[]= {0x01, 0x02, 0x04, 0x08, 0x10,0x20, 0x40, 0x80}; 
	uint8_t i;
	uint8_t	port_tmp =0;
	static uint8_t	col=0;
	uint8_t	shift;

	shift = 0x01 << col;	// Bitmaske erzeugen
				
	PORTC = 0xE0;			// Abschalten aller LED-Treiber
	PORTD = 0xFF;			// -> verhindert nachleuchten der LED's
	PORTB = 0xFF;

	for(i = 0; i < ROW_MAX; i++)
	{
		if(mask[col] & row[i])
			port_tmp &= ~LED_ROW[i];  // Ansteuerung invertiert durch PNP-Transistor
		else
			port_tmp |= LED_ROW[i];

		switch(i) // speichern der Portregisters
		{
			case 0:	// ROW 0
				PORTC = port_tmp;
				port_tmp = 0;	 
				break;
			case 5: // ROW 5
				PORTD = port_tmp;
				port_tmp = 0;
				break;
			case 8: // ROW 8
				PORTB = port_tmp;
				port_tmp = 0;
				break;
		}
	}

	PORTC |= shift;			// Einschalten der neuen Reihe
	
	col++;					// Nchste Reihe
	if(col >= 5)
		col = 0;

	dcf.msec++;				// Zeitgeber fr DCF-Empfang weiterdrehen
}	

